@@ -14,6 +14,7 @@ import (
 type Router struct {
 	Executer      cmd.Executer
 	ConfigManager *config.Manager
+	exitFunc      func(int)
 }
 
 // NewRouter creates a new Router with a config manager.
@@ -21,9 +22,19 @@ func NewRouter(e cmd.Executer, cm *config.Manager) *Router {
 	return &Router{
 		Executer:      e,
 		ConfigManager: cm,
+		exitFunc:      os.Exit,
 	}
 }
 
+// SetExitFunc overrides the default exit behavior (mainly for testing).
+func (r *Router) SetExitFunc(f func(int)) {
+	if f == nil {
+		r.exitFunc = os.Exit
+		return
+	}
+	r.exitFunc = f
+}
+
 // Route routes the command to the appropriate handler
 func (r *Router) Route(args []string) {
 	if len(args) == 0 {
@@ -60,7 +71,9 @@ func (r *Router) executeAlias(name string, args []string) {
 
 	case config.SequenceAlias:
 		if len(args) > 0 {
-			_, _ = fmt.Fprintf(os.Stderr, "Warning: arguments ignored for sequence alias '%s'\n", name)
+			_, _ = fmt.Fprintf(os.Stderr, "Error: sequence alias '%s' does not accept arguments (got %s)\n", name, strings.Join(args, " "))
+			r.exitFunc(1)
+			return
 		}
 
 		for _, c := range cmds {
@@ -77,7 +90,7 @@ type commandHandler func([]string)
 // getCommandHandlers returns a map of command names to their handlers
 func (r *Router) getCommandHandlers() map[string]commandHandler {
 	return map[string]commandHandler{
-		"help":       func(_ []string) { r.Executer.Help() },
+		"help":       func(args []string) { r.Executer.Help(args) },
 		"add":        r.Executer.Add,
 		"branch":     r.Executer.Branch,
 		"clean":      r.Executer.Clean,
@@ -106,6 +119,6 @@ func (r *Router) executeCommand(name string, args []string) {
 	if handler, exists := handlers[name]; exists {
 		handler(args)
 	} else {
-		r.Executer.Help()
+		r.Executer.Help(nil)
 	}
 }
