@@ -1,6 +1,10 @@
 package router
 
 import (
+	"bytes"
+	"io"
+	"os"
+	"strings"
 	"testing"
 
 	"github.com/bmf-san/ggc/v7/internal/testutil"
@@ -9,6 +13,7 @@ import (
 
 type mockExecuter struct {
 	helpCalled        bool
+	helpArgs          []string
 	branchCalled      bool
 	branchArgs        []string
 	commitCalled      bool
@@ -52,8 +57,9 @@ type mockExecuter struct {
 	interactiveCalled bool
 }
 
-func (m *mockExecuter) Help() {
+func (m *mockExecuter) Help(args []string) {
 	m.helpCalled = true
+	m.helpArgs = args
 }
 
 func (m *mockExecuter) Branch(args []string) {
@@ -160,6 +166,34 @@ func (m *mockExecuter) Interactive() {
 	m.interactiveCalled = true
 }
 
+func captureStderr(t *testing.T, fn func()) string {
+	t.Helper()
+
+	orig := os.Stderr
+	r, w, err := os.Pipe()
+	if err != nil {
+		t.Fatalf("failed to create pipe: %v", err)
+	}
+
+	os.Stderr = w
+
+	var buf bytes.Buffer
+	done := make(chan struct{})
+	go func() {
+		_, _ = io.Copy(&buf, r)
+		close(done)
+	}()
+
+	fn()
+
+	_ = w.Close()
+	os.Stderr = orig
+	<-done
+	_ = r.Close()
+
+	return buf.String()
+}
+
 func TestRouter(t *testing.T) {
 	cases := []struct {
 		name     string
@@ -173,6 +207,9 @@ func TestRouter(t *testing.T) {
 				if !m.helpCalled {
 					t.Error("Help should be called")
 				}
+				if len(m.helpArgs) != 0 {
+					t.Errorf("Help should receive no args, got %v", m.helpArgs)
+				}
 			},
 		},
 		{
@@ -458,6 +495,9 @@ func TestRouter(t *testing.T) {
 				if !m.helpCalled {
 					t.Error("Help should be called")
 				}
+				if len(m.helpArgs) != 0 {
+					t.Errorf("Help fallback should receive no args, got %v", m.helpArgs)
+				}
 			},
 		},
 		{
@@ -483,10 +523,12 @@ func TestRouter(t *testing.T) {
 
 func TestRouter_WithAliases(t *testing.T) {
 	cases := []struct {
-		name     string
-		aliases  map[string]interface{}
-		args     []string
-		validate func(t *testing.T, m *mockExecuter)
+		name             string
+		aliases          map[string]interface{}
+		args             []string
+		expectExit       bool
+		expectedExitCode int
+		validate         func(t *testing.T, m *mockExecuter)
 	}{
 		{
 			name: "simple alias",
@@ -535,13 +577,12 @@ func TestRouter_WithAliases(t *testing.T) {
 			aliases: map[string]interface{}{
 				"sync": []interface{}{"pull current", "push current"},
 			},
-			args: []string{"sync", "ignored"},
+			args:             []string{"sync", "ignored"},
+			expectExit:       true,
+			expectedExitCode: 1,
 			validate: func(t *testing.T, m *mockExecuter) {
-				if !m.pullCalled {
-					t.Error("Pull should be called for sequence alias")
-				}
-				if !m.pushCalled {
-					t.Error("Push should be called for sequence alias")
+				if m.pullCalled || m.pushCalled {
+					t.Error("Sequence alias should not run when arguments are provided")
 				}
 			},
 		},
@@ -575,12 +616,68 @@ func TestRouter_WithAliases(t *testing.T) {
 
 			m := &mockExecuter{}
 			r := NewRouter(m, configManager)
+			exitCalled := false
+			exitCode := 0
+			r.SetExitFunc(func(code int) {
+				exitCalled = true
+				exitCode = code
+			})
 			r.Route(tc.args)
 			tc.validate(t, m)
+			if tc.expectExit {
+				if !exitCalled {
+					t.Fatal("expected router to exit but it did not")
+				}
+				if exitCode != tc.expectedExitCode {
+					t.Fatalf("expected exit code %d, got %d", tc.expectedExitCode, exitCode)
+				}
+			} else if exitCalled {
+				t.Fatalf("unexpected exit with code %d", exitCode)
+			}
 		})
 	}
 }
 
+func TestRouter_SequenceAliasRejectsArguments(t *testing.T) {
+	mockClient := testutil.NewMockGitClient()
+	configManager := config.NewConfigManager(mockClient)
+	configManager.LoadConfig()
+
+	cfg := configManager.GetConfig()
+	cfg.Aliases = map[string]interface{}{
+		"deploy": []interface{}{"status"},
+	}
+
+	m := &mockExecuter{}
+	r := NewRouter(m, configManager)
+	exitCalled := false
+	exitCode := 0
+	r.SetExitFunc(func(code int) {
+		exitCalled = true
+		exitCode = code
+	})
+
+	output := captureStderr(t, func() {
+		r.Route([]string{"deploy", "production"})
+	})
+
+	if !exitCalled {
+		t.Fatal("sequence alias should exit when arguments are provided")
+	}
+	if exitCode != 1 {
+		t.Fatalf("expected exit code 1, got %d", exitCode)
+	}
+	if m.statusCalled {
+		t.Fatal("sequence commands should not execute when arguments are rejected")
+	}
+	if !strings.Contains(output, "sequence alias 'deploy' does not accept arguments") {
+		t.Fatalf("expected rejection message, got %q", output)
+	}
+	if !strings.Contains(output, "production") {
+		t.Fatalf("expected error to list offending arguments, got %q", output)
+	}
+}
+
 func TestRouter_ConfigManagerNil(t *testing.T) {
 	m := &mockExecuter{}
 	r := NewRouter(m, nil)
